import type { Term } from "@thi.ng/shader-ast";
/** @internal */
export declare const foldNode: import("@thi.ng/defmulti/api").MultiFn1<Term<any>, boolean | undefined>;
/**
 * Traverses given AST (potentially several times) and applies constant folding
 * optimizations where possible. Returns possibly updated tree (mutates
 * original).
 *
 * @remarks
 * Currently, only the following operations are supported / considered:
 *
 * - scalar math operators
 * - scalar math built-in functions
 * - single component vector swizzling
 * - literal hoisting
 *
 * @example
 * ```ts
 * import {
 *   add, defn, float, mul, neg, ret, scope, vec2
 *   $x, $y
 * } from "@thi.ng/shader-ast";
 * import { targetGLSL } from "@thi.ng/shader-ast-glsl";
 * import { constantFolding } from "@thi.ng/shader-ast-optimize";
 *
 * // function def
 * const foo = defn("float", "foo", ["float"], (x) => [
 *   ret(mul(x, add(neg(float(10)), float(42))))]
 * )
 *
 * const bar = vec2(100, 200);
 *
 * // program def
 * const prog = scope([
 *   foo,
 *   foo(add(float(1), float(2))),
 *   foo(add($x(bar), $y(bar)))
 * ], true);
 *
 * // serialized (GLSL)
 * const glsl = targetGLSL();
 * glsl(prog);
 *
 * // float foo(in float _sym0) {
 * //   return (_sym0 * (-10.0 + 42.0));
 * // };
 * // foo((1.0 + 2.0));
 * // foo((vec2(100.0, 200.0).x + vec2(100.0, 200.0).y));
 *
 * // with constant folding
 * glsl(constantFolding(prog))
 *
 * // float foo(in float _sym0) {
 * //   return (_sym0 * 32.0);
 * // };
 * // foo(3.0);
 * // foo(300.0);
 *
 * const expr = mul(float(4), $x(vec2(2)))
 *
 * glsl(expr)
 * // (4.0 * vec2(2.0).x)
 *
 * // optimize single expression
 * glsl(constantFolding(expr))
 * // 8.0
 * ```
 *
 * @param tree -
 */
export declare const constantFolding: (tree: Term<any>) => Term<any>;
//# sourceMappingURL=contant-folding.d.ts.map